;************************************************************************/
;*									*/
;*			Decode DCF77 Time Information			*/
;*									*/
;*              Author: Peter Dannegger                                 */
;*                      danni@specs.de                                  */
;*                                                                      */
;************************************************************************/
;used: r0, a0, zl, zh
;
DecodeDcf77:
	clt
	sbrs	flags, Dcf77sync		;second 00 detect
	rjmp	_ddc2
	sbrc	flags, Dcf77Error		;no Error
	rjmp	_ddc1
	cpi	NewSecond, 59                   ;right number of bits
	brne	_ddc1

	rcall	div60
	mov	reload, c0			;adjust internal RC timer
	mov	delta, z0			;fine adjust
	mov	prescaler, reload

	clr	Second				;load new time and date
	mov	Minute, NewMinute
	mov	Hour, NewHour
	mov	Day, NewDay
	mov	WeekDay, NewWeekDay
	mov	Month, NewMonth
	mov	Year, NewYear
	set					;display new time
_ddc1:
	clr	c0
	clr	c1
	ldi	NewSecond, 0
	cbr	flags, 1<<Dcf77Error^1<<dcf77sync^1<<dcf77parity
_ddc2:
	sbrs	flags, dcf77pulse
	rjmp	_ddc6

	mov	zl, NewSecond
	subi	zl, 21
	cpi	zl, (Dcf77TableEnd - Dcf77Table) * 2	;only bit 21 .. 58
	brcc	_ddc5

	ldi	a0, 1<<dcf77parity
	sbrc	flags, dcf77value
	eor	flags, a0			;parity ^= pulse

	ldi	zh, 0
	add	zl, zl				; * 2
	subi	zl, low(-(Dcf77Table * 2 + 1))
	sbci	zh, high(-(Dcf77Table * 2 + 1))
	lpm					;get value to add
	mov	a0, r0

	tst	a0
	brne	_ddc3
	sbrc	flags, dcf77parity
	sbr	flags, 1<<dcf77error		;set error bit
	rjmp	_ddc5
_ddc3:
	dec	zl
	lpm					;get byte address

	mov	zl, r0
	ld	r0, z
	cpi	a0, 1				;test a0 = 1
	brne	_ddc4
	clr	r0				;clear old value
_ddc4:	sbrc	flags, dcf77value
	add	r0, a0
	st	z, r0
_ddc5:
	cbr	flags, 1<<dcf77pulse^1<<dcf77value
	inc	newsecond
	brne	_ddc6
	sbr	flags, 1<<dcf77error
_ddc6:	ret
;-------------------------------------------------------------------------
Dcf77Table:
	.db	AddrNewMinute, 1		;minute
	.db	AddrNewMinute, 2
	.db	AddrNewMinute, 4
	.db	AddrNewMinute, 8
	.db	AddrNewMinute, 10
	.db	AddrNewMinute, 20
	.db	AddrNewMinute, 40
	.db	0, 0				;parity
	.db	AddrNewMinute + 1,  1		;hour
	.db	AddrNewMinute + 1,  2
	.db	AddrNewMinute + 1,  4
	.db	AddrNewMinute + 1,  8
	.db	AddrNewMinute + 1,  10
	.db	AddrNewMinute + 1,  20
	.db	0, 0				;parity
	.db	AddrNewMinute + 2,  1		;day
	.db	AddrNewMinute + 2,  2
	.db	AddrNewMinute + 2,  4
	.db	AddrNewMinute + 2,  8
	.db	AddrNewMinute + 2,  10
	.db	AddrNewMinute + 2,  20
	.db	AddrNewMinute + 3, 1		;weekday
	.db	AddrNewMinute + 3, 2
	.db	AddrNewMinute + 3, 4
	.db	AddrNewMinute + 4, 1		;month
	.db	AddrNewMinute + 4, 2
	.db	AddrNewMinute + 4, 4
	.db	AddrNewMinute + 4, 8
	.db	AddrNewMinute + 4, 10
	.db	AddrNewMinute + 5, 1		;year
	.db	AddrNewMinute + 5, 2
	.db	AddrNewMinute + 5, 4
	.db	AddrNewMinute + 5, 8
	.db	AddrNewMinute + 5, 10
	.db	AddrNewMinute + 5, 20
	.db	AddrNewMinute + 5, 40
	.db	AddrNewMinute + 5, 80
	.db	0, 0				;parity
Dcf77TableEnd:
;-------------------------------------------------------------------------
