;************************************************************************
;*									*
;*				SPI LCD Driver				*
;*									*
;*			Author: P. Dannegger				*
;*				danni@specs.de				*
;*									*
;************************************************************************
;
;optimized for smallest stack and register usage
;
;------------------------------------------------------------------------
;				Connection
;------------------------------------------------------------------------
;TINY12		74HCT164	LCD
;----------------------------------------
;PB0		8
;PB3				E
;PB4		1, 2		RS
;VCC		9, 14		VDD
;GND		7		VSS, R/W
;		3		D1
;		4		D3
;		5		D5
;		6		D7
;		10		D6
;		11		D4
;		12		D2
;		13		D0
;-------------------------------------------------------------------------
;				Initialize LCD
;-------------------------------------------------------------------------
;input: none
;output: none
;used: a0, a1, r0
;stack:	2
;
lcd_init:
	cbi	lcd_port, lcd_e
	in	a0, lcd_ddr
	sbr	a0,  1<<lcd_e^1<<lcd_ck^1<<lcd_rs
	out	lcd_ddr, a0
	ldi	a0, 1 + xtal * 150 / 2560000	;wait 15ms
	rcall	_wait256
	ldi	a0, 0x38			;2 lines 5*7
	rcall	lcd_command

	ldi	a0, 1 + xtal * 41 / 2560000	;wait 4.1ms
	rcall	_wait256
	ldi	a0, 0x38
	rcall	lcd_command

	ldi	a0, 1 + xtal * 1 / 2560000	;wait 100s
	rcall	_wait256
	ldi	a0, 0x38
	rcall	lcd_command

	ldi	a0, 0x08			;display off
	rcall	lcd_command

	ldi	a0, 0x01			;display clear
	rcall	lcd_command

	ldi	a0, 0x06
	rcall	lcd_command

	ldi	a0, 0x0C			;no cursor, no blink
;-------------------------------------------------------------------------
;				Write Command to LCD
;-------------------------------------------------------------------------
;input: a0 = command
;output: none
;used: a0, a1, r0
;stack:	1
;
lcd_command:
	clt
	rjmp	_lcd_out
;-------------------------------------------------------------------------
;				Write Data to LCD
;-------------------------------------------------------------------------
;input: a0 = data
;output: none
;used: a0, a1, r0
;stack:	1
;
lcd_data:
	set
;-------------------------------------------------------------------------
;internal subroutine
;
;input: a0 = byte
;
_lcd_out:
	mov	r0, a0
	ldi	a1, 8
_lou1:	cbi	lcd_port, lcd_ck
	cbi	lcd_port, lcd_rs
	cpi	a1, 4
	brcc	_lou2
	rol	r0
	rol	r0
	rjmp	_lou3
_lou2:	ror	r0
	sbrs	a1, 0
	sbrc	a1, 1
	ror	r0
_lou3:	brcc	_lou4
	sbi	lcd_port, lcd_rs
_lou4:	sbi	lcd_port, lcd_ck
	dec	a1
	brne	_lou1
	cbi	lcd_port, lcd_rs
	brtc	_lou5
	sbi	lcd_port, lcd_rs
_lou5:	sbi	lcd_port, lcd_e
	cbi	lcd_port, lcd_e

	brts	_wai2
	cpi	a0, 4
	brcc	_wai2
	ldi	a0, 1 + xtal * 16 / 2560000	;1.6ms
;-------------------------------------------------------------------------
;internal subroutine
;
_wait256:					;Tw = 256 cycle * a0
	clr	a1
_wai1:	subi	a1, 4
	sbci	a0, 0
	brne	_wai1
_wai2:	ret
;-------------------------------------------------------------------------
