;************************************************************************
;*									*
;*			Master Clock v1.2				*
;*									*
;*		Author: Christian Koch					*
;*			christian_koch@gmx.de				*
;*									*
;* Usage of the works is permitted provided that this			*
;* instrument is retained with the works, so that any entity		*
;* that uses the works is notified of this instrument.			*
;*									*
;* DISCLAIMER: THE WORKS ARE WITHOUT WARRANTY.				*
;*									*
;* [2004, Fair License; rhid.com/fair]					*
;*									*
;************************************************************************
;
;v1.0: slave clock pulse duration 2 seconds without pause
;v1.1 change: slave clock pulse duration 1 second, pause duration 1 second
;v1.2 change: easier modification of pulse and pause duration
;
;used:	a0
;
master_clock_init:
	in	a0, led_ddr
	sbr	a0, 1<<led_a^1<<led_b^1<<led_recv^1<<led_error^1<<led_valid
	out	led_ddr, a0

	cbi	led_port, PORTA7	;negative comparator input

	clr	a0
	out	EEAR, a0
	sbi	EECR, EERE		;read slave clock status from EEPROM
	in	slvhour, EEDR

	inc	a0
	out	EEAR, a0
	sbi	EECR, EERE
	in	slvminute, EEDR

	ldi	a0, 1<<ACBG^1<<ACIS1^1<<ACIS0	;enable analog comparator
	out	ACSR, a0
	ldi	a0, 1 + xtal * 10 / 2560000	;wait 1 ms for stab. Bandgap-Ref.
	rcall	_wait256
	sbi	ACSR, ACI		;clear analog comparator interrupt flag

	rjmp	master_clock_pulse	;repeat the last clock pulse
;-------------------------------------------------------------------------
;
master_clock:
	cpi	timeout, reload1sec	;pulse duration 1 second
	brlo	master_clock1
	ret				;pulse in progress

master_clock1:
	sbi	led_port, led_a		;switch off A
	sbi	led_port, led_b		;switch off B

	tst	timeout			;timeout == 0?
	breq	master_clock2
	ret				;pause in progress

master_clock2:
;	sbi	led_port, led_valid	;RTC starts on January 1st 2001
	cpi	year, 11		;time is valid if year >= 2011
	brsh	master_clock3
	ret				;time not valid

master_clock3:
	cbi	led_port, led_valid	;time valid

	mov	a0, hour		;convert to 12 hour values
	cpi	a0, 12
	brlo	master_clock4
	subi	a0, 12
master_clock4:
	sbic	switch_pin, switch_wait
	rjmp	master_clock5
	mov	slvhour, a0
	mov	slvminute, minute
	ret				;do not set the slave clock ahead

master_clock5:
	sbic	switch_pin, switch_forward
	rjmp	master_clock6
	rjmp	master_clock9

master_clock6:
	sub	a0, slvhour
	breq	master_clock8
	brsh	master_clock7
	subi	a0, -12
master_clock7:
	cpi	a0, 10			;time difference too great?
	brlo	master_clock9
	ret				;do not set the slave clock ahead

master_clock8:
	cp	slvminute, minute
	brlo	master_clock9
	ret				;nothing to do

master_clock9:
	cbi	ACSR, ACIE		;deactivate comparator interrupt (atomic)

	inc	slvminute		;set the slave clock ahead
	ldi	a0, 60
	cp	slvminute, a0
	brlo	master_clock_pulse

	clr	slvminute
	inc	slvhour
	ldi	a0, 12
	cp	slvhour, a0
	brlo	master_clock_pulse
	clr	slvhour

master_clock_pulse:
	sbi	ACSR, ACIE		;activate comparator interrupt

	ldi	timeout, 2 * reload1sec	;period = pulse + pause

;change to sbrs in the case the slave clock is permanently off by one minute
	sbrc	slvminute, 0		;test for even or odd
	rjmp	master_clock_pulse1
	cbi	led_port, led_a		;switch on A
	ret

master_clock_pulse1:
	cbi	led_port, led_b		;switch on B
	ret
;-------------------------------------------------------------------------
